

/* 
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */



#ifdef _WIN32
#include <windows.h>
#include <winbase.h>
#include <winsock.h>
#else
#include <errno.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#include <netdb.h>
#endif

#include "tcpsck.h"

#define MAX_ADDRESS 100
#define MAW_SOCKET  100


static int tcpsck_is_init = 0;

static struct sockaddr_in sin_tab[MAX_ADDRESS];
static int sin_idx;

static struct {
        tcpsck_return_t error_num;
        char *error_message;
        } tcpsck_error_message[] =
{
#ifdef _WIN32
 {WSAEINTR,"Interrupted system call"},
 {WSAEBADF,"Not a valid file descriptor"},
 {WSAEACCES,"The requested address is protected"},
 {WSAEFAULT,"Not a valid pointer"},
 {WSAEINVAL,"The socket has already been shut down"},
 {WSAEMFILE,"The per-process descriptor table is full"},
 {WSAEWOULDBLOCK,"The requested operation would block"},
 {WSAEINPROGRESS,"The connection cannot be completed immediately"},
 {WSAEALREADY,"Previous connection attempt has not yet completed"},
 {WSAENOTSOCK,"Not a socket descriptor"},
 {WSAEDESTADDRREQ,"No bind address was established"},
 {WSAEMSGSIZE,"A length in the msghdr structure is invalid"},
 {WSAEPROTOTYPE,"The type of socket and protocol do not match"},
 {WSAENOPROTOOPT,"The specified protocol is not supported"},
 {WSAEPROTONOSUPPORT,"The specified protocol is not supported"},
 {WSAESOCKTNOSUPPORT,"The specified address family is not supported"},
 {WSAEOPNOTSUPP,"The specified address family is not supported"},
 {WSAEPFNOSUPPORT,"The specified address family is not supported"},
 {WSAEAFNOSUPPORT,"The specified address family is not supported"},
 {WSAEADDRINUSE,"The address is already in use"},
 {WSAEADDRNOTAVAIL,"The specified address is not available on this machine"},
 {WSAENETDOWN,"The interface was not in the initialized state"},
 {WSAENETUNREACH,"The network is not reachable from this host"},
 {WSAENETRESET,"The network is not reachable from this host"},
 {WSAECONNABORTED,"A connection was forcibly closed by a peer"},
 {WSAECONNRESET,"A connection was forcibly closed by a peer"},
 {WSAENOBUFS,"No buffer space is available"},
 {WSAEISCONN,"The socket is already connected"},
 {WSAENOTCONN,"Receive on a socket that is not yet connected"},
 {WSAESHUTDOWN,"The socket has already been shut down"},
 {WSAETOOMANYREFS,"Too many references"},
 {WSAETIMEDOUT,"A timeout occurred"},
 {WSAECONNREFUSED,"The attempt to connect was forcefully rejected"},
 {WSAELOOP,"Loop detected"},
 {WSAENAMETOOLONG,"Name too long"},
 {WSAEHOSTDOWN,"The networking subsystem has not been started up"},
 {WSAEHOSTUNREACH,"The host is not reachable"},
 {WSAENOTEMPTY,"Not empty"},
 {WSAEPROCLIM,"Unknown error"},
 {WSAEUSERS,"Unknown error"},
 {WSAEDQUOT,"Unknown error"},
 {WSAESTALE,"Unknown error"},
 {WSAEREMOTE,"The host is not reachable"},
 {WSAEDISCON,"A connection was forcibly closed by a peer"},
#else
 {EINTR,"Interrupted system call"},
 {EBADF,"Not a valid file descriptor"},
 {ENOTSOCK,"Not a socket descriptor"},
 {EADDRNOTAVAIL,"The specified address is not available on this machine"},
 {EAFNOSUPPORT,"The specified address family is not supported"},
 {EALREADY,"Previous connection attempt has not yet completed"},
 {EISCONN,"The socket is already connected"},
 {EINVAL,"The socket has already been shut down"},
 {ETIMEDOUT,"Connection establishment timed ou"},
 {ECONNREFUSED,"The attempt to connect was forcefully rejected"},
 {ENETUNREACH,"The network is not reachable from this host"},
 {EADDRINUSE,"The address is already in use"},
 {EFAULT,"Not a valid pointer"},
 {EINPROGRESS,"The connection cannot be completed immediately"},
 {ENODEV,"Non-existent interface"},
 {ENOSPC,"All available virtual circuits are in use"},
 {ENETDOWN,"The interface was not in the initialized state"},
 {ENOBUFS,"No buffer space is available"},
 {EOPNOTSUPP,"Socket type does not support this call"},
 {EDESTADDRREQ,"No bind address was established"},
 {EACCES,"The requested address is protected"},
 {EHOSTDOWN,"The networking subsystem has not been started up"},
 {ESOCKTNOSUPPORT,"The specified address family is not supported"},
 {EPROTONOSUPPORT,"The specified protocol is not supported"},
 {EMFILE,"The per-process descriptor table is full"},
 {EPROTOTYPE,"The type of socket and protocol do not match"},
 {EWOULDBLOCK,"The requested operation would block"},
 {EMSGSIZE,"A length in the msghdr structure is invalid"},
 {ENOTCONN,"Receive on a socket that is not yet connected"},
 {ECONNRESET,"A connection was forcibly closed by a peer"},
 {EIO,"A timeout occurred"},
#endif
 {0,"Unknown error"}
};


tcpsck_return_t tcpsck_set_addr ( tcpsck_sock_addr_t *addr, 
                                  char *hostname, 
                                  int portnum )
{
  struct hostent *phe;
  struct sockaddr_in *psin;

  if (!tcpsck_is_init)
    {
      return (TCPSCK_ERROR);
    }

  psin = &(sin_tab[sin_idx]);
  sin_idx++;
  *addr = (tcpsck_sock_addr_t)psin;

  memset ((void *) psin, 0, sizeof (struct sockaddr_in));
  psin->sin_family = AF_INET;
  if (hostname == (char *)0)
    {
      psin->sin_addr.s_addr = INADDR_ANY;
    }
  else
    {
      phe = gethostbyname (hostname);
      if (phe == (struct hostent *)0)
	return (TCPSCK_ERROR);
      memcpy ((void *) &(psin->sin_addr), (void *) phe->h_addr,
              phe->h_length);
    }
  psin->sin_port = htons ((short)portnum);
  return (0);
}

tcpsck_return_t tcpsck_init ()
{
#ifdef _WIN32
  WSADATA   WSAData;
  
  sin_idx = 0;
  if (WSAStartup (MAKEWORD (1, 1), &WSAData) == 0)
    {
      tcpsck_is_init = 1;
      return (0);
    }
  else
    {
      return (TCPSCK_ERROR);
    }
#else
  sin_idx = 0;
  tcpsck_is_init = 1;
  return (0);
#endif

}

tcpsck_socket_handle_t tcpsck_new_socket ()
{
  tcpsck_socket_handle_t loc_sck;
  int on = 1;

  if (!tcpsck_is_init)
    return (TCPSCK_ERROR);
  if ((loc_sck = socket (AF_INET, SOCK_STREAM, 0)) == -1)
   return (TCPSCK_ERROR);

  if (setsockopt(loc_sck,SOL_SOCKET,
                 SO_REUSEADDR,(char *)&on,sizeof(int)) == -1)
   return (TCPSCK_ERROR);
  return (loc_sck);
}

tcpsck_return_t tcpsck_bind_and_listen ( tcpsck_socket_handle_t sock, 
                                         tcpsck_sock_addr_t *addr )
{
  if (bind (sock, (struct sockaddr *) *addr, 
                  sizeof (struct sockaddr_in)) == -1)
    {
      return (TCPSCK_ERROR);
    }
  if (listen (sock, 20) == -1)
    {
      return (TCPSCK_ERROR);
    }
  return (0);
}

tcpsck_return_t tcpsck_gethostname ( char *local_host, int len )
{
        return((tcpsck_return_t)gethostname(local_host, len));
}

tcpsck_return_t tcpsck_get_port ( tcpsck_socket_handle_t sock )
{
  struct sockaddr_in sin;
  int len = sizeof (struct sockaddr_in);

  if (getsockname (sock,(void *)&sin, &len) == -1)
   return(TCPSCK_ERROR);
  return (ntohs (sin.sin_port));
}

tcpsck_socket_handle_t tcpsck_accept ( tcpsck_socket_handle_t sock, 
                                       tcpsck_sock_addr_t *addr )
{
  int  len;
  len = sizeof (struct sockaddr_in);
  return ((tcpsck_socket_handle_t)
          accept (sock, (struct sockaddr *) *addr, &len));
}

tcpsck_return_t tcpsck_connect ( tcpsck_socket_handle_t sock, 
                                 tcpsck_sock_addr_t *addr )
{
  return ((tcpsck_return_t)
          connect (sock, (struct sockaddr *) *addr, 
                   sizeof (struct sockaddr_in)));
}

tcpsck_return_t tcpsck_close ( tcpsck_socket_handle_t sock )
{
#ifdef _WIN32
  return((tcpsck_return_t)closesocket (sock));
#else
  return((tcpsck_return_t)close (sock));
#endif
}

tcpsck_return_t tcpsck_send  ( tcpsck_socket_handle_t sock,
                               char *buffer,
                               int   buflen ) 
{
#ifdef __linux__
  return ((tcpsck_return_t)send (sock, buffer, buflen, MSG_NOSIGNAL));
#else
  return ((tcpsck_return_t)send (sock, buffer, buflen, 0));
#endif
}

tcpsck_return_t tcpsck_data_ready  ( tcpsck_socket_handle_t sock, int delay )
{
#ifdef _WIN32  
  struct fd_set l_set;
#else
  fd_set l_set;
#endif
  
  int l_nfd;
  struct timeval l_now;
  
  FD_ZERO(&l_set);
  FD_SET(sock,&l_set);
  l_now.tv_sec  = delay;
  l_now.tv_usec = 0;
  
#ifdef _WIN32
#define cast_fd_set (struct fd_set *)
#else
#define cast_fd_set (fd_set *)
#endif

  l_nfd = select(FD_SETSIZE,
		 cast_fd_set &l_set, 
		 cast_fd_set 0,
		 cast_fd_set 0, 
		 &l_now);

#undef cast_fd_set

  if ((l_nfd > 0) && FD_ISSET (sock, &l_set))
    {
      return (1);
    }
  else
    {
      return (0);
    }
}

tcpsck_return_t tcpsck_recv  ( tcpsck_socket_handle_t sock,
                               char *buffer,
                               int   buflen ) 
{
  
  memset(buffer,0,buflen);
  return ((tcpsck_return_t) recv (sock, (char *)buffer, buflen, 0));
  
}

void tcpsck_end ()
{
#ifdef _WIN32
  WSACleanup ();
  tcpsck_is_init = 0;
#endif
}

tcpsck_return_t tcpsck_get_last_error ( )
{
#ifdef _WIN32
 return((tcpsck_return_t) WSAGetLastError());
#else
 return((tcpsck_return_t)errno);
#endif
}

char *tcpsck_get_error_message ( tcpsck_return_t error_num )
{
 int idx = 0;
 
 while (tcpsck_error_message[idx].error_num != 0)
 {
  if (tcpsck_error_message[idx].error_num == error_num)
   break;
  idx++;
 }
 return(tcpsck_error_message[idx].error_message);
}





