

/*
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*
 * RTRT SystemTest training example
 *
 * file        : api.c
 * description : API definition to connect/disconnect, send/receive messages to/from the TCP/IP Server
 *
 */
#if defined(LINUX) || defined(__CYGWIN__)
#define UNIX
#endif
#include <stdio.h>
#include <stdlib.h>
#ifdef UNIX
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/un.h>
#include <netdb.h>
#include <arpa/inet.h>
#ifndef INADDR_NONE
#define INADDR_NONE -1
#endif
#endif
#ifdef _WIN32
#define ANSI_C
#include <windows.h>
#include <winbase.h>
#include <winsock.h>
#include <sys/timeb.h>
#define socket_close closesocket
#endif
#include <errno.h>
#ifdef _AIX
#include <sys/select.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/un.h>
#include <netdb.h>
#include <arpa/inet.h>
#ifndef INADDR_NONE
#define INADDR_NONE -1
#endif
#endif

#ifndef socket_close
#define socket_close close
#endif

#ifdef __cplusplus
#define EXTERN extern "C"
#else
#define EXTERN
#endif
#include "api.h"

char *err_string[]={
/*  0 */ "No error",
/*  1 */ "No message in the mailbox",
/*  2 */ "Error, connection closed by the server",
/*  3 */ "Error, try again",
/*  4 */ "Fatal error",
/*  5 */ "Error, ??",
/*  6 */ "Error, ??",
/*  7 */ "Error, connection refused",
/*  8 */ "Error, already initialized",
/*  9 */ "Error, not initialized",
"" };




int api_get_message (
#ifdef ANSI_C
                      appl_id_t  *id, void *message, int maxlen, int timeout)
#else
                      id, message,maxlen, timeout)
     appl_id_t  *id;
     void *message;
     int maxlen;
     int timeout;
#endif
{
  buffer_t  buffer;
  int       ret;
  fd_set    fds;
  struct timeval tv;


  /* If not init, error */
  if (!(id->init==1))  return(err_notinit);

  /* Put the socket id in the fd set */
  FD_ZERO(&fds);
  FD_SET(id->s_id,&fds);

  tv.tv_sec=timeout/1000;
  tv.tv_usec=(timeout%1000)*1000;

#ifdef hpux_9
      select(FD_SETSIZE,(int *)&fds,NULL,NULL,&tv);
#else
      select(FD_SETSIZE,&fds,NULL,NULL,&tv);
#endif

  /* If no fd is set : there is no message to read */
  if (!FD_ISSET(id->s_id,&fds)) return(err_empty);

  /* In the case the fd is set, read the message */
  ret = recv (id->s_id, buffer, maxlen,0);
  if (ret == 0) return (err_closed);
  if (ret == -1)
   if (errno == EINTR) return(err_again);
   else return(err_fatal);

  /* Copy the received message */
  memcpy(message, &buffer, maxlen);

  return (err_ok);
}



int api_send_message (
#ifdef ANSI_C
                      appl_id_t *id, const void *message, int len)
#else
                      id, message,len)
     appl_id_t *id;
     const void *message;
     int len;
#endif
{
  buffer_t  buffer;
  int       ret;

  if (!(id->init==1))  return(err_notinit);

/* Copy the message to be sent */
memcpy(&buffer, message, len);
  ret = send (id->s_id, buffer, len,0);
if (ret == -1)
   if (errno == EINTR) return(err_again);
   else return(err_fatal);
  return (err_ok);
}



int api_init(
#ifdef ANSI_C
                 appl_id_t *id,char *server_ip_address)
#else
                 id,server_ip_address)
 appl_id_t *id;
 char *server_ip_address;
#endif
{
#ifdef _WIN32
  WSADATA   WSAData;
#endif
 int local_id;
 struct sockaddr_in local_mbx;
 int ret;
 struct hostent *hptr;
 unsigned int addr=0;
 char hostname[256];
 if (id->init==1)  return(err_already);

#ifdef _WIN32
 if (WSAStartup (MAKEWORD (1, 1), &WSAData) != 0)
 {
  exit(-1);
 }
#endif

 memset ((void *) &local_mbx, 0, sizeof(struct sockaddr_in));

 if(!server_ip_address)
 {
   /* server is located on the same machine */
   ret = gethostname (hostname, 255);
   if (ret == -1) return (err_connect);
   server_ip_address=hostname;
 }
 hptr = gethostbyname (server_ip_address);
 if (hptr == NULL) return (err_connect);
 memcpy ((void *) &local_mbx.sin_addr, hptr->h_addr, hptr->h_length);
 local_mbx.sin_family = hptr->h_addrtype;
 local_mbx.sin_port = (unsigned short)(htons(SERVER_PORT_NUM));

 /* Create a new socket */
 local_id = socket (AF_INET, SOCK_STREAM, 0);
 if (local_id<0) return(err_fatal);

 /* Connect it to the Server port */
 ret = connect(local_id, (struct sockaddr *)&local_mbx, sizeof(struct sockaddr_in));
 if (ret!=0) return(err_connect);
 id->init = 1;

 /* Stores the new socket in the id parameter */
 id->s_id = local_id;

 return(err_ok);
}


int api_end(
#ifdef ANSI_C
                   appl_id_t *id)
#else
                    id)
appl_id_t *id;
#endif
{
 if (!(id->init==1))  return(err_notinit);
 socket_close(id->s_id);
 id->init=0;
 return(err_ok);
}










