

/* 
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2010 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*
 * RTRT SystemTest SUT example
 *
 * description : broadCastServer
 *
 */
#if defined(LINUX) || defined(__CYGWIN__)
#define UNIX
#endif

#include <stdio.h>
#include <stdlib.h>
#ifdef UNIX
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <signal.h>
#include <sys/types.h>
/*#include <sys/un.h>*/
#include <netdb.h>
#endif
#ifdef _WIN32
#include <windows.h>
#include <winbase.h>
#include <winsock.h>
#include <sys/timeb.h>
#include <signal.h>
#include <process.h>
#endif
#include <string.h>
#include <errno.h>
#ifdef _AIX
#include <sys/select.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <signal.h>
#include <sys/types.h>
/*#include <sys/un.h>*/
#include <netdb.h>
#endif

/* C type used for a buffer */
#define MAX_BUFFER_LENGTH   1500
typedef unsigned char buffer_t[MAX_BUFFER_LENGTH];
/* port number of the PING SERVER application */
#ifndef DEFAULT_PORT_NUM
#define DEFAULT_PORT_NUM 1906
#endif
#define MAX(a,b) ((a)>(b)?((a)+1):((b)+1))
#define trace printf
#define trace1(a) printf("[%s] ",get_time());printf(a)
#define trace2(a,b) printf("[%s] ",get_time());printf(a,b)
#define trace3(a,b,c) printf("[%s] ",get_time());printf(a,b,c)
#define trace4(a,b,c,d) printf("[%s] ",get_time());printf(a,b,c,d)
#define MAX_APPL  25

char *get_time ()
{
	static char tmp[255];
#ifdef UNIX
	struct timeval tv;
	struct timezone tzp;

	gettimeofday(&tv,&tzp);
	sprintf(tmp,"%02d:%02d.%02d",(tv.tv_sec/60)%60,
			tv.tv_sec%60,tv.tv_usec/10000);
#endif
#ifdef _WIN32
	struct timeb t;

	ftime(&t);
	sprintf(tmp,"%02d:%02d.%02d",(t.time/60)%60,
			t.time%60,t.millitm/10);
#endif
	return(tmp);
}

void handler (int foo)
{
	printf("Server shut down!! signal %d\n",foo);
	exit (0);
}
#ifndef SERVTIME
#define SERVTIME 20
#endif

/*
	SERVER
	usage : server [<port> [<time>]]
		  The ping server can be launched with one (optional) parameter which is the TCP port
 */

int main (
#ifdef ANSI_C
		int argc, char * argv[])
#else
argc, argv)
		int argc;
		char * argv[];
#endif
		{
#ifdef _WIN32
			WSADATA   WSAData;
#endif
			char      hostname[256];
			struct hostent *hptr;
			int       local_id;
			struct sockaddr_in local_mbx;
			int       nfds;
			fd_set    readfds;
			fd_set    savefds;
			fd_set    local_set;
			int       ret,rets;
			int       new_id,sender;
			int connected=0;
			struct timeval timeout;
			struct in_addr localIp;

			buffer_t  buffer;

			int       ping_client_number = 0;
			int       port = 0;
			int       duration = 0;
			char		loopback=1;
			setbuf (stderr, NULL);
			setbuf (stdout, NULL);

			trace("Server running with pid = %d\n",getpid());
#ifdef UNIX
			signal (SIGINT, handler);
			signal (SIGQUIT, handler);
#endif
#ifdef _WIN32
			signal (SIGINT,handler);
			signal (SIGTERM, handler);
			signal (SIGABRT, handler);
			signal (SIGBREAK, handler);

			signal (SIGILL, handler);
			signal (SIGFPE, handler);
			signal (SIGSEGV, handler);

			if (WSAStartup (MAKEWORD (1, 1), &WSAData) != 0)
			{
				exit(-1);
			}
#endif

			ret = gethostname (hostname, 255);
			if (ret == -1)
			{
				perror ("Error, gethostname");
				exit (1);
			}
			hptr = gethostbyname (hostname);
			if (hptr == NULL)
			{
				perror ("Error, gethostbyname");
				exit (1);
			}
			/* to display IP address of the server machine */
			memcpy ((void *) &localIp, hptr->h_addr, hptr->h_length);

			if(argc > 1)
			{
				port = atoi (argv[1]);
				if(argc >2)
				{
					duration =atoi (argv[2]);
					if(argc >3)
					{
						if(!strncmp("-noloopback",argv[3],4))
							loopback=0;
					}
				}
			}
			if(port==0) port = DEFAULT_PORT_NUM;
			if(duration ==0) duration = SERVTIME;


			FD_ZERO (&savefds);
			FD_ZERO (&local_set);

			nfds = 0;


			/*
			 * server connection socket
			 */
			memset ((void *) &local_mbx, 0, sizeof(struct sockaddr_in));
			/*  memcpy ((void *) &local_mbx.sin_addr, hptr->h_addr, hptr->h_length); */
			local_mbx.sin_addr.s_addr=INADDR_ANY; /* no IP address filter */

			local_mbx.sin_family = hptr->h_addrtype;
			local_mbx.sin_port = htons(port);

			local_id = socket (AF_INET, SOCK_STREAM, 0);

			ret = bind (local_id, (struct sockaddr *)&local_mbx, sizeof (struct sockaddr_in));

			if (ret == -1)
			{
				perror ("Error, local bind");
				exit (1);
			}

			ret = listen (local_id, 20);
			if (ret == -1)
			{
				perror ("Error, local listen");
				exit (1);
			}

			FD_SET (local_id, &savefds);
			nfds = MAX (nfds, local_id);

			trace("Server socket created at port number %d\n",port);

			timeout.tv_usec = 100000;

			trace1("Configuration completed\n");
			trace ("Server up and running on %s\n",hostname);
#ifdef _WIN32
			trace ("Server IP : %d.%d.%d.%d\n",localIp.S_un.S_un_b.s_b1,
					localIp.S_un.S_un_b.s_b2,
					localIp.S_un.S_un_b.s_b3,
					localIp.S_un.S_un_b.s_b4);
#endif

			for (;;)
			{
				int       nb;

				timeout.tv_sec = duration;
				memcpy (&readfds, &savefds, sizeof (fd_set));

#ifdef hpux_9
				nb = select (nfds, (int *) &readfds, NULL, NULL, &timeout);
#else
				nb = select (nfds, &readfds, NULL, NULL, &timeout);
#endif
				if (nb == -1)
				{
					perror ("Error, select");
					exit (1);
				}
				if (nb == 0)
				{
					fprintf (stderr, "Timeout: server ending\n");
					exit (0);
				}
				if (nb > 0)
				{
					/*
					 * local connection indication
					 */
					if (FD_ISSET (local_id, &readfds))
					{
						size_t       size = sizeof (struct sockaddr_in);

						new_id = accept (local_id, (void *) &local_mbx, &size);
						if (new_id == -1)
						{
							perror ("Error, local accept");
							exit (1);
						}
						FD_SET (new_id, &savefds);
						FD_SET (new_id, &local_set);
						trace2 ("connection from %d\n", new_id);
						connected++;
						nfds = MAX (nfds, new_id);
						continue;
					}


					for (new_id = 0; new_id < nfds; new_id++)
						if (FD_ISSET (new_id, &readfds))
							break;
					/*
					 * local data indication
					 */
					if ((new_id < nfds))
					{
						ret = recv (new_id, buffer, MAX_BUFFER_LENGTH,0);

						if (ret == 0 || ret==-1)
						{
							FD_CLR (new_id, &local_set);
							FD_CLR (new_id, &savefds);
							trace2 ("Disconnecting from %d\n", new_id);
							connected--;
							close (new_id);
						}
						else
						{
							char sent=0;
							sender=new_id;
							trace3 ("Receive Message from %d length = %d sent to",new_id, ret);
							for (new_id = 0; new_id < nfds; new_id++)
								if (((sender != new_id) ||(loopback)) && FD_ISSET (new_id, &local_set))
								{
									rets = send (new_id, buffer, ret,0);
									sent=1;
									trace(" %d",new_id);
									if (rets == -1)
									{
										fprintf (stderr, "[%d] ", new_id);
										perror ("Error, write");
										exit (1);
									}
								}
							if(!sent)
							{
								printf("[%s]",buffer);
							}
							trace("\n");
							if(!strncmp((const char *)buffer,"***KILL SERVER MSG***",21))
							{
								fprintf (stderr, "Kill message received: server ending\n");
								exit (0);
							}
						}
					}
				}
			}
		}




