

/*
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


#include "file.h"

const char *EMPTY_LIST = "list is empty";

File::File (const File &src) : first (0), last(0)
{
   ListItem *last_it = 0;
   Counter = 0;
   for (ListItem *it = src.first; it; it = it->next)
   {
      ListItem *new_it = new ListItem;
      new_it->str = it->str;
      new_it->previous = last_it;
      new_it->next = 0;
      if (last_it == 0)
         first = new_it;
      else
	last_it->next = new_it;
      last_it = new_it;
   }
   last = last_it;
}

File::~File ()
{
   clear ();
}

void File::removed (ListItem *item)
{
	Counter--;
}

int File::count () const
{
   int result = 0;
   ListItem *it = first;

   while (it != 0)
   {
      result++;
      it = it->next;
   }

   return result;
}

void File::insert (const char *str)
{
   ListItem *item = new ListItem;

   item->str = str;
   item->previous = 0;
   item->next = first;
   if (last == 0) last = item;
   else first->previous = item;
   first = item;
   Counter++;
}

void File::append (const char *str)
{
   ListItem *item = new ListItem;

   item->str = str;
   item->next = 0;
   item->previous = last;
   if (first == 0) first = item;
   else last->next = item;
   last = item;
   Counter++;
}

const char *File::popHead ()
{
   if (first == 0) throw InvalidOperation (EMPTY_LIST);

   const char *result = first->str;
   ListItem *it = first;

   removed (first);
   first = first->next;
   delete it;
   if (first == 0) last = 0;
   else first->previous = 0;
   return result;
}

const char *File::popTail ()
{
   if (first == 0) throw InvalidOperation (EMPTY_LIST);

   const char *result = last->str;
   ListItem *it = last;

   removed (last);
   last = last->previous;
   delete it;
   if (last == 0) first = 0;
   else last->next = 0;
   return result;
}

void File::clear ()
{
   ListItem *it;
   while (first != 0)
   {
      removed (first);
      it = first;
      first = first->next;
      delete it;
   }
   last = 0;
}
