
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------


---------------------------------------------------------------------
--                      PACKAGE HISTO BODY                         --
---------------------------------------------------------------------

package body histo is

procedure evaluate_histo(x1,y1,x2,y2:in integer;desc: out t_desc) is
  local_min   : t_pixel := t_pixel'last;
  local_max   : t_pixel := t_pixel'first;
  local_ave   : float   := 0.0;
  local_histo : t_histo := (others => 0);
  sygma       : integer := 0;

begin

-- overflow test on x
  if (not (x1 in Bitmap'range(1)))or (not (x2 in Bitmap'range(1)))
     then
       message(1..13)   := "OVERFLOW ON X";
       message(14..Message'last) := (others => ' ');
       raise Overflow;
     end if;

--  overflow test on y
  if (not (y1 in Bitmap'range(2)))or (not (y2 in Bitmap'range(2)))
     then
       message(1..13) := "OVERFLOW ON Y";
       message(14..Message'last) := (others => ' ');
       raise Overflow;
     end if;

  message(message'range) := (others => ' ');

-- loop on bitmap sub-window for histogram calculation
  for i in x1..x2 loop
    for j in y1..y2 loop

      -- min and max memorisation
      if local_min>Bitmap(i,j) then local_min:= Bitmap(i,j); end if;
      if local_max<Bitmap(i,j) then local_max:= Bitmap(i,j); end if;

      -- histogram evaluation
      local_histo(Bitmap(i,j)) := local_histo(Bitmap(i,j))+1;

    end loop; -- for j in y1..y2
  end loop; -- for i in x1..x2

-- average calculation
  for i in t_histo'range loop
    local_ave := Local_ave + float( i * local_histo(i) );
    sygma     := sygma + local_histo(i);
  end loop;
  if ( sygma = 0 ) then
    desc.average := 0.0;
  else
    desc.Average := local_ave / float(sygma);
  end if;

-- min and max return
  desc.extreme.min := local_min;
  desc.extreme.max := local_max;
  desc.histogram := local_histo;

end Evaluate_histo;

end histo;


