

/* 
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


//////////////////////////////////////////////////////////////////////
// Shape.h: interface for the Shape class.
//////////////////////////////////////////////////////////////////////

#include "Point.h"
#include "GraphicEngine.h"


#ifndef SHAPE_H
#define SHAPE_H

//------------------------------------
// abstract class Shape
// the class Shape is an abstract class which describes the main features of
// a drawing component.
// 
//
//The computing the perimeter is a lazy evaluation, i.e. it is only computed if itis needed.
//Then the getPerimeter method is a getter of the perimeter value and  computates the value.
//the state value ot the perimeter is contained in the value itself :
// -1 means the value have ever been calculated
// 0 means the value have been calculated but a modification has put the value in an obsolete state
// other positive value means that the perimeter is well computed.
// 
// the resetPerimeter is called when a modification of perimeter data is done. Then the value is obsolete
//
// the computePerimeter method is an abstract method which the programmer has to specify in the sub classes.
// This method is call by getPerimeter method in order to update the value of the perimeter.
//------------------------------------
class Shape  
{
public:
        static const  signed OBSOLETE  ;
        static const  signed NOT_CALCULATED ;

        Shape(const Point& p) : theOrg(p), theGrEn(),thePerimeter(NOT_CALCULATED){}

        void    translate(int dx, int dy);
        void    translate (const Point& p);
        Point   getOrigin() const;

        // getter for perimeter
        double getPerimeter () ;

        virtual ~Shape(){}
        virtual void    print() const = 0;
        virtual void display() const {theGrEn.draw();}

private :
        Point   theOrg;



protected :

         GraphicEngine theGrEn ;
         double thePerimeter ;  


        // reset the perimeter to zero.
        void resetPerimeter(); 

        // compute  the perimeter and update the variable perimeter
        virtual void computePerimeter()=0 ;
        
        
        
};

//------------------------------------
// exception class
//------------------------------------

class BadParameter 
{
   public :
          BadParameter() {}
};


#endif // SHAPE_H

