

/* 
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/* 
 * (C) 2003 Rational Software Corporation. All Rights Reserved.  
 * 
 * file        : stack.c
 *
 */

#include <stdio.h>
#include <stdlib.h>
#ifdef UNIX
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/un.h>
#include <netdb.h>
#endif
#ifdef WIN32
#include <windows.h>
#include <winbase.h>
#include <winsock.h>
#include <sys/timeb.h>
#include <process.h>
#endif
#include <string.h>
#include <errno.h>


#include "stack.h"
#include "atlprobe.h" 



#define MAX(a,b) ((a)>(b)?((a)+1):((b)+1))
#define trace1(a) printf("[%s] ",get_time());printf(a)
#define trace2(a,b) printf("[%s] ",get_time());printf(a,b)
#define trace3(a,b,c) printf("[%s] ",get_time());printf(a,b,c)
#define trace4(a,b,c,d) printf("[%s] ",get_time());printf(a,b,c,d)
#define MAX_APPL  25

#ifdef WIN32
#define close closesocket
#endif

int       remote_id;
 
typedef struct
    {
      int       port;
      int       connected;
      int       id;
      char     *hostname;
} A_REMOTE_CLIENT;

A_REMOTE_CLIENT remote_stack_tab[10];
char hostname[256];


char *get_time ()
{
char tmp[255];
#ifdef UNIX
struct timeval tv;
struct timezone tzp;

  gettimeofday(&tv,&tzp);
  sprintf(tmp,"%02d:%02d.%02d",(tv.tv_sec/60)%60,
          tv.tv_sec%60,tv.tv_usec/10000);
#endif
#ifdef WIN32
struct timeb t;

  ftime(&t);
  sprintf(tmp,"%02d:%02d.%02d",(t.time/60)%60,
          t.time%60,t.millitm/100);
#endif
  return(strdup(tmp));
}

void nothing (int foo)
{
exit (0);
}

void handler (int foo)
{
atl_end_trace(atl_stack);
atl_dump_trace();
exit (0);
}

int       get_message (int id, message_t * message)
{
  buffer_t  buffer;
  int       ret,i;
  for(i=0;i<MAX_BUFFER_LENGTH;i++)
    buffer[i]=0;
  ret = recv (id, buffer, MAX_BUFFER_LENGTH,0);
  if (ret == -1)
    return (0);
  /*
    {
      fprintf (stderr, "[%d] ", id);
      perror ("Error, read");
      exit (1);
    }
    */
  if (ret == 0)
    return (0);
  buf_message (&buffer, message);
  return (ret);
}

int       send_message (int id, message_t * message)
{
  buffer_t  buffer;
  int       ret,i;
  for(i=0;i<MAX_BUFFER_LENGTH;i++)
    buffer[i]=0;
  message_buf (message, &buffer);
  ret = send (id, buffer, MAX_BUFFER_LENGTH,0);
  return (ret);
}

int main ()
{
#ifdef WIN32
  WSADATA   WSAData;
#endif
  int       i;
  int       nb;
  int       id;
  int       ret;
  int       nfds;
  int       new_id;
  int       local_id;
  int       notconnected;
  int       remote_stack_num;
  struct hostent *hptr;
  struct sockaddr_in local_mbx;
  struct sockaddr_in remote_mbx;
  struct sockaddr_in loc_sock;
  fd_set    readfds;
  fd_set    savefds;
  fd_set    local_set;
  fd_set    remote_set;
  char     *stack_local;
  FILE     *stack_conf;
  struct timeval timeout;
  message_t message;
  protocol g_protocol; 
  
  struct
  {
      char     *name;
    int       id;
  }
  rmt_appl[MAX_APPL], loc_appl[MAX_APPL];
  int       rmt_appl_number = 0;
  int       loc_appl_number = 0;
  int       stack_num;
  int       duration = 1;
  int       GlobalTimeOut = 60;
  int retry;
  
  setbuf (stderr, NULL);
  setbuf (stdout, NULL);
  trace2("Stack running at pid = %d\n",getpid());
  trace1("Reading configuration ...\n");

#ifdef WIN32

  if (WSAStartup (MAKEWORD (1, 1), &WSAData) != 0)
  {
          exit(-1);
  }
#endif

  ret = gethostname (hostname, 255);
  if (ret == -1)
    {
      perror ("Error, gethostname");
      exit (1);
    }
  
  /* Caution ! gethostbyname does not like '.' in hostname */
  {
    char * point = strchr(hostname, '.');
    if (point) *point = '\0';
  }
  
  hptr = gethostbyname (hostname);
  if (hptr == NULL)
    {
      perror ("Error, gethostbyname");
      exit (1);
    }

#ifdef ATL_WITHOUT_STUDIO
  stack_local = getenv ("STACK_LOCAL");
  if (stack_local == NULL)
  {
      fprintf (stderr, "Error, STACK_LOCAL not set\n");
      exit (1);
  }
  stack_num = atoi (stack_local);
  trace2("Identification number = %d\n",stack_num);
  if ((stack_conf = fopen ("./stackcnf.txt", "rt")) == NULL)
  {
      perror ("Error, file ./stackcnf.txt");
      exit (1);
  }
  notconnected = 0;
  remote_stack_num = 0;
  while (!feof (stack_conf))
  {
      int       portnum;

      fscanf (stack_conf, "%s %d\n", hostname, &portnum);

      remote_stack_tab[remote_stack_num].hostname = strdup(hostname);
      remote_stack_tab[remote_stack_num].port = portnum;
      if (remote_stack_num == stack_num)
        remote_stack_tab[remote_stack_num].connected = 1;
      else
        remote_stack_tab[remote_stack_num].connected = 0;
      ++remote_stack_num;

      notconnected = 1;
  }
  fclose (stack_conf);

#else
  remote_stack_num = 1;
  stack_num = 0;
  trace2("Identification number = %d\n",stack_num);
  remote_stack_tab[0].hostname = strdup("localhost");
  remote_stack_tab[0].port = 1928;
  remote_stack_tab[0].connected = 1;
  notconnected = 1;
#endif

  FD_ZERO (&savefds);
  FD_ZERO (&local_set);
  FD_ZERO (&remote_set);
  nfds = 0;
  /* 
   * local connexion service mail box
   */
  memset ((void *) &local_mbx, 0, sizeof(struct sockaddr_in));
  memcpy ((void *) &local_mbx.sin_addr, hptr->h_addr, hptr->h_length);
  local_mbx.sin_family = hptr->h_addrtype;
  local_mbx.sin_port = htons(LOCAL_PORT_NUM);

  local_id = socket (AF_INET, SOCK_STREAM, 0);

  ret = bind (local_id, (struct sockaddr *)&local_mbx, sizeof (struct sockaddr_in));

  if (ret == -1)
    {
      perror ("Error, local bind");      exit (1);
    }

  ret = listen (local_id, 20);
  if (ret == -1)
    {
      perror ("Error, local listen");
      exit (1);
    }

  FD_SET (local_id, &savefds);
  nfds = MAX (nfds, local_id);

  trace2("Application mailbox created at port number %d\n",LOCAL_PORT_NUM);

  trace2("Creating stack mailbox on port %d ...\n",
          remote_stack_tab[stack_num].port);
  /* 
   * remote connexion service mail box
   */
  memset ((void *) &remote_mbx, 0, sizeof(struct sockaddr_in));
  memcpy ((void *) &remote_mbx.sin_addr, hptr->h_addr, hptr->h_length);
  remote_mbx.sin_family = hptr->h_addrtype;
  remote_mbx.sin_port = htons( remote_stack_tab[stack_num].port );

  remote_id = socket (AF_INET, SOCK_STREAM, 0);

  atl_start_trace(atl_stack,"",stack,100000);
  atl_format_trace("atl_format.hts");
  retry = 1;
  while (retry)
    {
      retry = 0;
      ret = bind (remote_id, (struct sockaddr *)&remote_mbx, sizeof (remote_mbx));
      if (ret == -1)
        {
#ifdef UNIX
          if (errno == EADDRINUSE)
            {
              retry = 1;
              fprintf (stderr,"Will retry remote bind in 5 sec");
              sleep (5);
            }
          else
#endif
            {
              perror ("Error, remote bind");
              exit (1);
            }
        }
    }
  ret = listen (remote_id, 20);
  if (ret == -1)
    {
      perror ("Error, remote listen");
      exit (1);
    }

  trace1("Stack mailbox created\n");

  FD_SET (remote_id, &savefds);
  nfds = MAX (nfds, remote_id);

  timeout.tv_usec = stack_num * 100000;

  trace1("Configuration completed\n");
  trace2 ("Stack up and running on %s\n",hostname);
  


 for (;;)
    {
 
      timeout.tv_sec = duration;
      memcpy (&readfds, &savefds, sizeof (fd_set));

      nb = select (nfds, &readfds, NULL, NULL, &timeout);

      if (nb == -1)
        {
          perror ("Error, select");
          exit (1);
        }
/*	  trace1("."); */
	  if(GlobalTimeOut-- ==0)
	   {
		 trace1("Process Time out.");
		exit(1);
          }
      if ((nb == 0) && notconnected)
        {

          notconnected = 0;
          for (i = 0; i < remote_stack_num; ++i)
            {
              if ((i != stack_num) && (!remote_stack_tab[i].connected))
                {
                  trace4 ("[channel %d] : Sending T_CONNECT_RQ to %s on port %d\n",
                          i,
                          remote_stack_tab[i].hostname,
                          remote_stack_tab[i].port);

		  /* Caution ! gethostbyname does not like '.' in hostname */
		  {
		    char * point = strchr(remote_stack_tab[i].hostname, '.');
		    if (point) *point = '\0';
		  }

                  hptr = gethostbyname (remote_stack_tab[i].hostname);
                  if (hptr == NULL)
                    {
                      perror ("Error, gethostbyname");
                      exit (1);
                    }
                  memcpy ((void *) &loc_sock.sin_addr, hptr->h_addr, hptr->h_length);
                  loc_sock.sin_family = hptr->h_addrtype;
                  loc_sock.sin_port = htons( remote_stack_tab[i].port );

                  id = socket (AF_INET, SOCK_STREAM, 0);
                  ret = connect (id, (struct sockaddr *)&loc_sock, sizeof (struct sockaddr_in));

                  if (ret == 0)
                    {
                      message_t message;
		      init_message(&message);
                      trace2 ("Receive T_CONNECT_CF from %s\n",
                              remote_stack_tab[i].hostname);
                      FD_SET (id, &savefds);
                      FD_SET (id, &remote_set);
                      nfds = MAX (nfds, id);
                      remote_stack_tab[i].connected = 1;
                      remote_stack_tab[i].id = id;
                      message.type = IDENTIFICATION;
                      message.num = stack_num;
                      trace2 ("Sending IDENTIFICATION_RQ to %d\n", id);
                      send_message (id, &message);
                    }
                  else
                    {
                      trace2 ("Receive T_CONNECT_ABORT from %s\n",
                              remote_stack_tab[i].hostname);
                      notconnected = 1;
                      close (id);
                    }
                }
            }
          if (notconnected && (duration < 10))
            ++duration;
        }
      if (nb > 0)
        {
          GlobalTimeOut =60;
          /*  
           * local connection indication
           */
          if (FD_ISSET (local_id, &readfds))
            {
              int       size = sizeof (struct sockaddr_in);

              new_id = accept (local_id, (void *) &local_mbx, &size);
              trace2 ("Receive T_CONNECT_ID from : %d\n", new_id);
              trace2 ("Sending T_CONNECT_RS to : %d\n", new_id);
		
              if (new_id == -1)
                {
                  perror ("Error, local accept");
                  exit (1);
                }
              FD_SET (new_id, &savefds);
              FD_SET (new_id, &local_set);
              nfds = MAX (nfds, new_id);
            }

          /*  
           * remote connection indication
           */
          if (FD_ISSET (remote_id, &readfds))
            {
              int       size = sizeof (struct sockaddr_in);

              new_id = accept (remote_id, (void *) &remote_mbx, &size);
              trace2 ("Receive T_CONNECT_ID from : %d\n", new_id);
              trace2 ("Sending T_CONNECT_RS to : %d\n", new_id);
              if (new_id == -1)
                {
                  perror ("Error, remote accept");
                  exit (1);
                }
              FD_SET (new_id, &savefds);
              FD_SET (new_id, &remote_set);
              nfds = MAX (nfds, new_id);
            }

          for (new_id = 0; new_id < nfds; new_id++)
            if (FD_ISSET (new_id, &readfds))
              break;

          /*
           * local data indication
           */
          if ((new_id < nfds) && (FD_ISSET (new_id, &local_set)))
            {
              ret = get_message (new_id, &message);           
   if (ret == 0)
                {
                  FD_CLR (new_id, &local_set);
                  FD_CLR (new_id, &savefds);
                  trace2 ("Receive T_DISCONNECT_ID from %d\n", new_id);
                  trace2 ("Sending T_DISCONNECT_RS to %d\n", new_id);
                  close (new_id);
                }
              else
                {
		
	
		  switch (message.type)
                    {
		      case SHUTDOWN:
                      {
			
			atl_recv_trace(atl_stack,remote_stack,&message,message_t,message);		 		
			trace2 ("Receive SHUTDOWN from %s\n",message.applname);
			if(strcmp("DUMPTRACE",message.applname)==0)
			  {
			    atl_end_trace(atl_stack);
			    atl_dump_trace();
			    exit (0);
			  }
#ifdef UNIX
			signal (SIGINT, nothing);
			signal (SIGQUIT, nothing);
			signal (SIGTERM, nothing);
#endif
			break;
			}
                    case REGISTER:
                      {
                        int       i;
			atl_select_trace(atl_stack,ATL_protocol_select,REGISTER);
			atl_recv_trace(atl_stack,remote_stack,&g_protocol,protocol,stack_response);		                         
			trace2 ("Receive REGISTER_ID from %s\n",
                                 message.applname);
                        
			loc_appl[loc_appl_number].name
                          = strdup (message.applname);
                        loc_appl[loc_appl_number].id = new_id;
                        ++loc_appl_number;
                        for (i = 0; i < remote_stack_num; ++i)
                          {
                            if ((i != stack_num) &&
                                (remote_stack_tab[i].connected))
                              {
                                message_t message;
				init_message(&message);
                                message.type = REGISTER;
                                strcpy (message.applname,
                                        loc_appl[loc_appl_number - 1].name);
                                trace2 ("Sending REGISTER_RQ to %d\n",
                                        remote_stack_tab[i].id);
                                send_message (remote_stack_tab[i].id, &message);
				atl_select_trace(atl_stack,ATL_protocol_select,REGISTER);
				atl_send_trace(atl_stack,remote_stack,&g_protocol,protocol,stack_response);

                              }
                          }
                        if (loc_appl_number < MAX_APPL)
                          {
			    message_t ack;                        
			    init_message(&ack);
			    trace2 ("Sending REGISTER_RS to %s\n", 
                                    message.applname);
			
                            ack.type = ACK;
                            send_message (new_id, &ack);
		  	    atl_select_trace(atl_stack,ATL_protocol_select,ACK);
			    atl_send_trace(atl_stack,appli_emet,&g_protocol,protocol,stack_response);
                          }
                        else
                          {
                            trace2 ("Sending REGISTER_ABORT to %s\n", 
                                    message.applname);
                            message.type = NEG_ACK;
                            send_message (new_id, &message);
		  	    atl_select_trace(atl_stack,ATL_protocol_select,NEG_ACK);
			    atl_send_trace(atl_stack,appli_emet,&g_protocol,protocol,stack_response);

                          }
                        break;
                      }
                    case UNREGISTER:
                      {
                        int       i,
                                  j;
			
                        for (i = 0; i < loc_appl_number; ++i)
                          if (!strcmp (loc_appl[i].name,
                                       message.applname))
                            break;
                        if (i < loc_appl_number)
                          {
                            trace2 ("Receive UNREGISTER_ID from %s\n",
                                 message.applname);
                            for (j = 0; j < remote_stack_num; ++j)
                              {
                                if ((j != stack_num) &&
                                    (remote_stack_tab[j].connected))
                                  {
                                    message_t message;
				    init_message(&message);
                                    message.type = UNREGISTER;
                                    strcpy (message.applname,
                                            loc_appl[i].name);
                                    trace2 ("Sending REGISTER_RQ to %d\n",
                                        remote_stack_tab[j].id);
                                    send_message (remote_stack_tab[j].id,
                                                  &message);
		  		

                                  }
                              }
                            free (loc_appl[i].name);
                            loc_appl[i].name = "";
                            loc_appl[i].id = 0;
                            trace2 ("Sending UNREGISTER_RS to %s\n", 
                                    message.applname);
                            message.type = ACK;
                            send_message (new_id, &message);
			   

                          }
                        break;
                      }
                    case DATA:
                      {
                        int       i;
			atl_recv_trace(atl_stack,remote_stack,&message,message_t,message);
			for (i=0;i<loc_appl_number;i++)
			  if(loc_appl[i].id==new_id)
			    break;
			
                        trace3 ("Receive DATA_ID from %s to %s\n", 
                                loc_appl[i].name, message.applname);
			
                        for (i = 0; i < loc_appl_number; ++i)
                          if (!strcmp (loc_appl[i].name,
                                       message.applname))
                            break;
                        if (i < loc_appl_number)
                          {
                            message_t ack;
			    init_message(&ack);
                            trace2 ("Sending DATA_RS to %d\n", 
                                    new_id);
                            ack.type = ACK;
                            send_message (new_id, &ack);
			    atl_select_trace(atl_stack,ATL_protocol_select,ACK);
			    atl_send_trace(atl_stack,appli_emet,&g_protocol,protocol,stack_response);

                            trace2 ("Sending DATA_RQ to %d\n", 
                                    loc_appl[i].id);
                            send_message (loc_appl[i].id, &message);
			    atl_send_trace(atl_stack,appli_recept,&message,message_t,message);

                          }
                        else
                          {

                            for (i = 0; i < rmt_appl_number; ++i)
                              if (!strcmp (rmt_appl[i].name,
                                           message.applname))
                                break;
                            if (i < rmt_appl_number)
                              {
                                message_t ack;
				init_message(&ack);
                                trace2 ("Sending DATA_RS to %s\n", 
                                    message.applname);
                                ack.type = ACK;
                                send_message (new_id, &ack);
				atl_select_trace(atl_stack,ATL_protocol_select,ACK);
				atl_send_trace(atl_stack,remote_stack,&g_protocol,protocol,stack_response);
				trace2 ("Sending DATA_RQ to %d\n", rmt_appl[i].id);
                                send_message (rmt_appl[i].id, &message);
				
                              }
                            else
                              {
                                message_t ack;
				init_message(&ack);
                                trace2 ("Sending DATA_ABORT to %s\n", 
                                    message.applname);
                                ack.type = NEG_ACK;
                                send_message (new_id, &ack);
				atl_select_trace(atl_stack,ATL_protocol_select,NEG_ACK);
		  		atl_send_trace(atl_stack,appli_emet,&g_protocol,protocol,stack_response);

                              }
                          }
                        break;
                      }
                     default:
                      {
                        trace2 ("Unknown message type %d\n",message.type);
                        break;
                      }
                    }
                }
            }

          /*
           * remote data indication
           */
          if ((new_id < nfds) && (FD_ISSET (new_id, &remote_set)))
            {
              ret = get_message (new_id, &message);
              if (ret == 0)
                {
                  int       i;

                  FD_CLR (new_id, &local_set);
                  FD_CLR (new_id, &savefds);
                  trace2 ("Receive T_DISCONNECT_ID from %d\n", new_id);
                  close (new_id);
                  trace2 ("Sending T_DISCONNECT_RS to %d\n", new_id);
                  for (i = 0; i < remote_stack_num; ++i)
                    if (remote_stack_tab[i].id == new_id)
                      break;

                  if (remote_stack_tab[i].id == new_id)
                    {
                      remote_stack_tab[i].connected = 0;
                      notconnected = 1;
                      duration = 1;
                    }
                }
              else
                {
		  

                  switch (message.type)
                    {
                    case ACK:
                      {
                        trace2 ("Receive IDENTIFICATION_CF from %d\n",
                                 message.num);
                        break;
                      }
                    case IDENTIFICATION:
                      {
			atl_recv_trace(atl_stack,remote_stack,&message,message_t,message);
			trace2 ("Receive IDENTIFICATION_ID from %d\n",
                                 message.num);
                        remote_stack_tab[message.num].id = new_id;
                        remote_stack_tab[message.num].connected = 1;
                        message.type = ACK;
                        message.num = stack_num;
                        trace2 ("Sending IDENTIFICATION_RS to %d\n",
                                 new_id);
                        send_message (new_id, &message);
			atl_select_trace(atl_stack,ATL_protocol_select,ACK);
			atl_send_trace(atl_stack,remote_stack,&g_protocol,protocol,stack_response);

                        break;
                      }
                    case REGISTER:
                      {
			atl_select_trace(atl_stack,ATL_protocol_select,REGISTER);
			atl_recv_trace(atl_stack,remote_stack,&g_protocol,protocol,stack_response);
			trace2 ("Receive REGISTER_ID from %s\n", 
                                message.applname);
                        rmt_appl[rmt_appl_number].name =
                          strdup (message.applname);
                        rmt_appl[rmt_appl_number].id = new_id;
                        ++rmt_appl_number;
                        if (rmt_appl_number > MAX_APPL)
                          exit (1);
                        break;
                      }
                    case UNREGISTER:
                      {
                        int       i;
			for (i = 0; i < rmt_appl_number; ++i)
                          if (!strcmp (rmt_appl[i].name,
                                       message.applname))
                            break;
                        if (i < rmt_appl_number)
                          {
                            trace2 ("Receive UNREGISTER_ID from %s\n", 
                                message.applname);
                            free (rmt_appl[i].name);
                            rmt_appl[i].name = "";
                            rmt_appl[i].id = 0;
                          }
                        break;
                      }
                     default:
                      {
                        trace2 ("Unknown message type %d\n",message.type);
                        break;
                      }
                    }
                }
            }
        }
    }
}







