

/* 
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/* 
 * (C) 2003 Rational Software Corporation. All Rights Reserved.  
 * 
 * file        : stack.h
 * description : definition of C types
 *
 */

#ifndef __STACK__
#define __STACK__

#include "errdef.h"

/* port number for local connection */
#define LOCAL_PORT_NUM 1910

/* C types used for messages */

#define MAX_USERDATA_LENGTH 256
#define MAX_BUFFER_LENGTH   400
#define SIZE_APPLICATION_NAME 20
/* message types */
#define IDENTIFICATION 4         /* stack private use only       */
#define REGISTER       0         /* stack private use only       */
#define UNREGISTER     1         /* stack private use only       */
#define DATA           5         /* data message                 */
#define ACK            2         /* acknowledge message          */
#define NEG_ACK        3         /* negative acknowledge message */
#define SHUTDOWN       6
/* C type of a message */
typedef struct s_message
{
  int type;                                /* type of the message           */
  int num;                                 /* stack identification number   */
  char from[SIZE_APPLICATION_NAME];                            /* application sender name       */
  char applname[SIZE_APPLICATION_NAME];                        /* application name              */
  char userdata[MAX_USERDATA_LENGTH];      /* user data                     */
  int  length;                             /* user data length              */
} message_t;


typedef union u_protocol
{
  char registered[1];
  char unregistered[1];
  char ack[1];
  char negack[1];
} protocol; 

/* C type used for a buffer */
typedef unsigned char buffer_t[MAX_BUFFER_LENGTH];

/* C type used for application identification */
typedef struct {
 int s_id;
 char applname[SIZE_APPLICATION_NAME];
 int stack;
} appl_id_t;

/*
 * Functions Prototypes
 * return code are defined in file errdef.h
 */

/* function for encoding a message into a buffer (private use) */
int message_buf(message_t *message,buffer_t *buffer);

/* function for decoding a buffer into a message (private use) */
int buf_message(buffer_t *buffer,message_t *message);

/* function to init the API */
int mbx_init(appl_id_t *id);

/* function to register a application onto the local stack */
int mbx_register(appl_id_t *id);

/* function to get a message from the application mailbox  */
int mbx_get_message (appl_id_t  *id, message_t *message, int timeout);

/* function to send a message to another application mailbox  */
int mbx_send_message (appl_id_t *id, message_t *message);

/* function to unregister from the local stack  */
int mbx_unregister(appl_id_t *id);

/* function to end the API */
int mbx_end(appl_id_t *id);

void init_message(message_t* msg);
#endif



