

/* 
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/* 
 * (C) 2003 Rational Software Corporation. All Rights Reserved.  
 * 
 * file        : stackapi.c
 * description : stack API
 *
 */

#include "stack.h"
#include <stdio.h>
#include <stdlib.h>
#ifdef UNIX
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/un.h>
#include <netdb.h>
#endif
#ifdef WIN32
#include <windows.h>
#include <winbase.h>
#include <winsock.h>
#include <sys/timeb.h>
#endif
#include <errno.h>
#ifdef _AIX
#include <sys/select.h>
#endif



#ifdef WIN32
#define close closesocket
#endif


static int init = 0;

int mbx_get_message (appl_id_t  *id, message_t *message, int timeout)
{
  buffer_t  buffer;
  int       ret;
#ifdef WIN32
  struct fd_set    fds;
#else
  fd_set    fds;
#endif
  struct timeval tv;

  if (!init)  return(err_notinit);
  FD_ZERO(&fds);
  FD_SET(id->s_id,&fds);

  tv.tv_sec=timeout;
  tv.tv_usec=0;

  select(FD_SETSIZE,&fds,NULL,NULL,&tv);

  if (!FD_ISSET(id->s_id,&fds)) return(err_empty);
  ret = recv (id->s_id, buffer, MAX_BUFFER_LENGTH,0);
  if (ret == 0) return (err_closed);
  if (ret == -1) 
   {
     if (errno == EINTR) return(err_again);
     else return(err_fatal);
   }
  buf_message (&buffer, message); 
  return (err_ok);
}

int mbx_send_message (appl_id_t *id, message_t *message)
{
  buffer_t  buffer;
  int       ret;
  
  if (!init)  return(err_notinit);
  strcpy(message->from,id->applname);
  message_buf (message, &buffer);
  ret = send (id->s_id, (char *)buffer, MAX_BUFFER_LENGTH,0);
  if (ret == -1) 
   {
     if (errno == EINTR) return(err_again);
     else return(err_fatal);
   }
  return (err_ok);
}

int mbx_init(appl_id_t *id)
{
#ifdef WIN32
  WSADATA   WSAData;
#endif
  int local_id;
  struct sockaddr_in local_mbx;
  int ret;
  char hostname[256];
  struct hostent *hptr;

  if (init)  return(err_already);
  
#ifdef WIN32
  if (WSAStartup (MAKEWORD (1, 1), &WSAData) != 0)
  {
    exit(-1);
  }
#endif
 ret = gethostname (hostname, 255);
 if (ret == -1) return (err_connect);

 /* Caution ! gethostbyname does not like '.' in hostname */
 {
   char * point = strchr(hostname, '.');
   if (point) *point = '\0';
 }

 hptr = gethostbyname (hostname);
 if (hptr == NULL) return (err_connect);
 
 memset ((void *) &local_mbx, 0, sizeof(struct sockaddr_in));
 memcpy ((void *) &local_mbx.sin_addr, hptr->h_addr, hptr->h_length);
 local_mbx.sin_family = hptr->h_addrtype;
 local_mbx.sin_port = htons(LOCAL_PORT_NUM);
 
 local_id = socket (AF_INET, SOCK_STREAM, 0);
 if (local_id<0) return(err_fatal);
 ret = connect(local_id, (struct sockaddr *)&local_mbx, sizeof(struct sockaddr_in));
 if (ret!=0) return(err_connect);
 init = 1;
 id->s_id = local_id;
 return(err_ok);
}


int mbx_register(appl_id_t *id)
{
 int ret;
 message_t message;

 if (!init)  return(err_notinit);
 message.type=REGISTER;
 strcpy(message.applname,id->applname);
 ret = mbx_send_message(id, &message);
 if (ret != err_ok) return(ret);
 ret = mbx_get_message(id, &message,5);
 if (ret == err_empty) return(err_connect);
 if (ret != err_ok) return(ret);
 if (message.type != ACK) return(err_fatal);
 return(err_ok);
}

int mbx_unregister(appl_id_t *id)
{
  int ret;
  message_t message;
  
  if (!init)  return(err_notinit);
  /* read all remaining messages */
 do {
   ret = mbx_get_message(id, &message,3);
 } while (ret==err_ok);
 message.type=UNREGISTER;
 strcpy(message.applname,id->applname);
 ret = mbx_send_message(id, &message);
 if (ret != err_ok) return(ret);
 ret = mbx_get_message(id, &message,5);
 if (ret == err_empty) return(err_connect);
 if (ret != err_ok) return(ret);
 if (message.type != ACK) return(err_fatal);
 return(err_ok);
}

int mbx_end(appl_id_t *id)
{
  if (!init)  
    return(err_notinit);
  
  close(id->s_id);
  init=0;
  return(err_ok);
}

