

/*
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */



/* TestSuiteC - Code Coverage example */

#include "Types.h"
#define MAX(a,b) (a>b?a:b)
#define CHECK(a) if(a<0) return 0
#ifndef BITWISE
#define _OR_ ||
#define _AND_ &&
#else /* BITWISE */
#define _OR_ |
#define _AND_ &
#endif /* BITWISE */


UInt_t          Max_Index;
Detect_Switch_t set_switch(Detect_Switch_t);
Detect_Switch_t Detect_switch_case;
Switch_t        Select_case;
UInt_t          Global_Detect_While;

extern UInt_t   Table[];


int SimpleCondition(int x)
{
 /* This procedure consists of 3 statement blocks and no implicit blocks.
    The first statement block is the entire function.
    The second statement block is the true block after the "if" statement.
    The third statement block is the false block after the "else" statement.
    There are no implicit blocks as there are statement blocks for each true and false case of every decision.
    The report indicates 3 statement blocks and 3 decision blocks.
    The decision block column is the sum of the statement blocks + implicit blocks, i.e. 3 + 0 = 3
  */
 if (x<0)
 {
  return(-x);
 }
 else
 {
  return(x);
 }
}

void call()
{
 int x=1;
 x=SimpleCondition(x);
}

int SimpleConditionWithImplicit(int x)
{
 /* This procedure consists of 3 statement blocks and 1 implicit blocks.
    The first statement block is the entire function.
    The second statement block is the true block after the "if" statement.
    The third statement block is the block just before the final return statement.
    There is 1 implicit blocks as there is no "else" clause for the "if" statement.
    The report indicates 3 statement blocks and 4 decision blocks.
    The decision block column is the sum of the statement blocks + implicit blocks, i.e. 3 + 1 = 4
  */
 if (x<0)
 {
  return(-x);
 }
 return(x);
}

int AndCondition(int x)
{
 /* This procedure consists of 3 statement blocks and no implicit blocks.
    The first statement block is the entire function.
    The second statement block is the true block after the "if" statement.
    The third statement block is the false block after the "else" statement.
    There are no implicit blocks as there are statement blocks for each true and false case of every decision.
    The report indicates 3 statement blocks and 3 decision blocks.
    The decision block column is the sum of the statement blocks + implicit blocks, i.e. 3 + 0 = 3

    There are 2 conditions that need to be tested true and false which gives 4 basic conditions.
    There are 2 modified condtions as there is one operator && 
    There are 3 multiple conditions which correspond to the truth table TT, TF, FX
  */
 if ((x>0) _AND_ (x<100))
 {
  return(x);
 }
 else return(0);
}

int OrCondition(int x)
{
 /* This procedure consists of 3 statement blocks and no implicit blocks.
    The first statement block is the entire function.
    The second statement block is the true block after the "if" statement.
    The third statement block is the false block after the "else" statement.
    There are no implicit blocks as there are statement blocks for each true and false case of every decision.
    The report indicates 3 statement blocks and 3 decision blocks.
    The decision block column is the sum of the statement blocks + implicit blocks, i.e. 3 + 0 = 3

    There are 2 conditions that need to be tested true and false which gives 4 basic conditions.
    There are 2 modified condtions as there is one operator && 
    There are 3 multiple conditions which correspond to the truth table TX, FF, FT
  */
 if ((x==1) _OR_ (x>10))
 {
  return(x);
 }
 else return(0);
}

int Loop(int x)
{
 /* This procedure consists of 4 statement blocks and 1 implicit blocks.
    The first statement block is the entire function.
    The second statement block is the true block after the "if" statement in the CHECK macro.
    The third statement block is after the CHECK macro as this macro contains a "return" statement
    The fourth statement block is the "for" loop block.
    There is 1 implicit blocks as there is no "else" clause for the "if" statement in the CHECK macro.
    The report indicates 4 statement blocks and 5 decision blocks.
    The decision block column is the sum of the statement blocks + implicit blocks, i.e. 4 + 1 = 5

    There are no basic conditions.

    There 3 loop blocks as each "for" loop has to be executed 0, 1 and 2 or more times.
  */
 int idx;
 CHECK(x);
 for (idx=(x-1); idx>0; idx--)
 {
  x+=(x-idx);
 }
 return(x);
}

UInt_t Check_statement_coverage (Boolean_t Cond1, Boolean_t Cond2,Boolean_t Cond3,Boolean_t Cond4,Boolean_t Cond5)
{
 /* This procedure consists of 10 statement blocks and 3 implicit blocks.
    There are 6 loop blocks.
    There are 22 basic conditions, 5 modified conditions and 10 multiple conditions

    Using the information in the comments above and the information in the documentation find them yourself
    Don't forget case blocks count as one bloc, if there is no default then there is an implicit block.
    Ternary expressions introduce 2 blocs, even in macros, and at least one condtion
  */


  UInt_t Return_Value=0;
  UInt_t Index=11;
  Boolean_t Bool_Res;
  UInt_t bitwiseResult=0;

 /*Code to test instrumentation of if instruction and implicit else */
 if (Cond1==TRUE)
 {
    Return_Value=1;
 }

 /*Code to test instrumentation of case instruction and implicit default */

 switch (Select_case)
 {
    case CASE_1:
        Detect_switch_case= set_switch(DETECT_CASE_1);
    break;

    case CASE_2:
        Detect_switch_case= set_switch(DETECT_CASE_2);
    break;

    case CASE_3:
        Detect_switch_case= set_switch(DETECT_CASE_3);
    break;

 }

 /*bitwise operation on non boolean operator is not a logical operation
  So no MCDC are computed */
 bitwiseResult= Cond5 | Cond3 | (UInt_t)0x46;
 if(bitwiseResult > 0x46)
 {
     bitwiseResult=0;
 }

 /*Code to test instrumentation of  MCDC */
 Bool_Res = Cond1 _OR_ Cond2 _AND_ Cond3;

 if ( ( (Cond1 _AND_ Cond2) _OR_ (Cond3 _AND_ Cond4) ) _AND_ (Cond5) )
 {
     Return_Value=MAX(0xff,Index);
 }


 /*Code to test instrumentation of "while" instruction */

 while (Global_Detect_While!=0)
 {
     Global_Detect_While--;
 }


 /*Code to test instrumentation of "for" instruction */

 for (Index=0; Index<Max_Index;Index++)
 {
     Table[Index]= Index;
 }

 return ( Return_Value);

}



