

/*
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */



/* TestSuiteC - Code Coverage example */

#include "Types.h"
#include "my_bool.h"
#define MAX(a,b) (a>b?a:b)
#define CHECK(a) if(a<0) return 0
#ifndef BITWISE
#define _OR_ ||
#define _AND_ &&
#else /* BITWISE */
#define _OR_ |
#define _AND_ &
#endif /* BITWISE */


UInt_t          Max_Index;
Detect_Switch_t set_switch(Detect_Switch_t);
Detect_Switch_t Detect_switch_case;
Switch_t        Select_case;
UInt_t          Global_Detect_While;

UInt_t Table[20];
extern Detect_Switch_t set_switch(Detect_Switch_t p1);

class CodeCoverage
{
public:
	CodeCoverage() {proI=30;priI=15;priB=true;proB=false;Global_Detect_While=0;Select_case=CASE_1;};
	~CodeCoverage() {};
	int SimpleCondition(int x);
	int SimpleConditionWithImplicit(int x);
	int AndCondition(int x);
	int AndOperator(bool x, bool y);
	int OrCondition(int x);

	int Loop(int x);
	UInt_t Check_statement_coverage (bool Cond1, bool Cond2,bool Cond3,bool Cond4,bool Cond5) ;

	int pubI;
	bool pubB;
private:
	int priI;
	bool priB;
protected:
	int proI;
	bool proB;
};

int CodeCoverage::SimpleCondition(int x)
{
 /* This procedure consists of 3 statement blocks and no implicit blocks.
    The first statement block is the entire function.
    The second statement block is the true block after the "if" statement.
    The third statement block is the false block after the "else" statement.
    There are no implicit blocks as there are statement blocks for each true and false case of every decision.
    The report indicates 3 statement blocks and 3 decision blocks.
    The decision block column is the sum of the statement blocks + implicit blocks, i.e. 3 + 0 = 3
  */
 if (x<0)
 {
  return(-x);
 }
 else
 {
  return(x);
 }
}

int CodeCoverage::SimpleConditionWithImplicit(int x)
{
 /* This procedure consists of 3 statement blocks and 1 implicit blocks.
    The first statement block is the entire function.
    The second statement block is the true block after the "if" statement.
    The third statement block is the block just before the final return statement.
    There is 1 implicit blocks as there is no "else" clause for the "if" statement.
    The report indicates 3 statement blocks and 4 decision blocks.
    The decision block column is the sum of the statement blocks + implicit blocks, i.e. 3 + 1 = 4
  */
 if (x<0)
 {
  return(-x);
 }
 return(x);
}

int CodeCoverage::AndCondition(int x)
{
 /* This procedure consists of 3 statement blocks and no implicit blocks.
    The first statement block is the entire function.
    The second statement block is the true block after the "if" statement.
    The third statement block is the false block after the "else" statement.
    There are no implicit blocks as there are statement blocks for each true and false case of every decision.
    The report indicates 3 statement blocks and 3 decision blocks.
    The decision block column is the sum of the statement blocks + implicit blocks, i.e. 3 + 0 = 3

    There are 2 conditions that need to be tested true and false which gives 4 basic conditions.
    There are 2 modified condtions as there is one operator &&
    There are 3 multiple conditions which correspond to the truth table TT, TF, FX
  */
 if ((x>0) _AND_ (proI<100))
 {
  proI=x;
  return(x);
 }
 else return(0);
}

int CodeCoverage::AndOperator(bool x, bool y)
{
  my_bool a(x);  my_bool b(y);
  if (a && b)
  {
    return 1;
  }
  else
  {
	return 0;
  }
}

int CodeCoverage::OrCondition(int x)
{
 /* This procedure consists of 3 statement blocks and no implicit blocks.
    The first statement block is the entire function.
    The second statement block is the true block after the "if" statement.
    The third statement block is the false block after the "else" statement.
    There are no implicit blocks as there are statement blocks for each true and false case of every decision.
    The report indicates 3 statement blocks and 3 decision blocks.
    The decision block column is the sum of the statement blocks + implicit blocks, i.e. 3 + 0 = 3

    There are 2 conditions that need to be tested true and false which gives 4 basic conditions.
    There are 2 modified condtions as there is one operator &&
    There are 3 multiple conditions which correspond to the truth table TX, FF, FT
  */
 if ((x==1) _OR_ (priI>10))
 {
  priI=x;
  return(x);
 }
 else return(0);
}

int CodeCoverage::Loop(int x)
{
 /* This procedure consists of 4 statement blocks and 1 implicit blocks.
    The first statement block is the entire function.
    The second statement block is the true block after the "if" statement in the CHECK macro.
    The third statement block is after the CHECK macro as this macro contains a "return" statement
    The fourth statement block is the "for" loop block.
    There is 1 implicit blocks as there is no "else" clause for the "if" statement in the CHECK macro.
    The report indicates 4 statement blocks and 5 decision blocks.
    The decision block column is the sum of the statement blocks + implicit blocks, i.e. 4 + 1 = 5

    There are no basic conditions.

    There 3 loop blocks as each "for" loop has to be executed 0, 1 and 2 or more times.
  */
 int idx;
 CHECK(x);
 for (idx=(x-1); idx>0; idx--)
 {
  x+=(x-idx);
 }
 return(x);
}

UInt_t CodeCoverage::Check_statement_coverage (bool Cond1, bool Cond2,bool Cond3,bool Cond4,bool Cond5)
{
 /* This procedure consists of 10 statement blocks and 3 implicit blocks.
    There are 6 loop blocks.
    There are 22 basic conditions, 5 modified conditions and 10 multiple conditions

    Using the information in the comments above and the information in the documentation find them yourself
    Don't forget case blocks count as one bloc, if there is no default then there is an implicit block.
    Ternary expressions introduce 2 blocs, even in macros, and at least one condtion
  */


  UInt_t Return_Value=0;
  UInt_t Index=11;
  bool Bool_Res;
  UInt_t bitwizeResult=0;

 /*Code to test instrumentation of if instruction and implicit else */
 if (Cond1==true)
 {
    Return_Value=1;
    proB= Cond1 _AND_ Return_Value ? Cond1 : true;
 }

 /*Code to test instrumentation of case instruction and implicit default */

 switch (Select_case)
 {
    case CASE_1:
        Detect_switch_case= set_switch(DETECT_CASE_1);
		Select_case=CASE_3;
    break;

    case CASE_2:
        Detect_switch_case= set_switch(DETECT_CASE_2);
    break;

    case CASE_3:
        Detect_switch_case= set_switch(DETECT_CASE_3);
		//Global_Detect_While++;
    break;

 }

 /*bitwize operation on non boolean operator is not a logical operation
  So no MCDC are computed */
 bitwizeResult= Cond5 | Cond3 | (UInt_t)0x46;
 if(bitwizeResult > 0x46)
 {
     bitwizeResult=0;
 }

 /*Code to test instrumentation of  MCDC */
 Bool_Res = proB _OR_ Cond2 _AND_ Cond3;

 if ( ( (proB _AND_ Cond2) _OR_ (Cond3 _AND_ Cond4) ) _AND_ (Cond5) )
 {
     Return_Value=MAX(0xff,Index);
 }


 /*Code to test instrumentation of "while" instruction */

 while (Global_Detect_While!=0)
 {
     Global_Detect_While--;
 }

 if ( priB ) Global_Detect_While++;
 priB = !(Global_Detect_While == 1);

 /*Code to test instrumentation of "for" instruction */

 for (Index=0; Index<Max_Index;Index++)
 {
     Table[Index]= Index;
 }

 return ( Return_Value);

}

#if 0
int main()
{
	int x;
	CodeCoverage cov;

	cov.SimpleCondition(-1);
	cov.SimpleCondition(+1);
	cov.SimpleConditionWithImplicit(-1);
	cov.SimpleConditionWithImplicit(+1);
	cov.AndCondition(50);
	cov.AndCondition(-1);
	cov.AndCondition(200);
	cov.OrCondition(20);
	cov.OrCondition(1);
	cov.OrCondition(5);
	cov.Loop(1);
	cov.Loop(2);
	cov.Loop(3);
	cov.Check_statement_coverage(false,false,false,false,false);
	cov.Check_statement_coverage(false,false,true, true, true);
	cov.Check_statement_coverage(false,false,true, false,false);
	cov.Check_statement_coverage(true, true, true, true, true);
	cov.Check_statement_coverage(true, true, true, true, false);
	cov.Check_statement_coverage(true, false,true, false,false);

	return 0;
}
#endif


