

/*
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


#ifdef _WIN32
#include <windows.h>
#else
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#endif

/* TestSuiteC - Memory Profiling example */
#define trace printf
#ifndef trace
#define trace // //no trace
#endif

#define RTRT_LONGJMP  /* undefine RTRT_LONGJMP if longjump is not available in this TDP */
#define ALLOC(s) malloc(s)
#define CALLOC(n,s) calloc(n,s)
#define REALLOC(p,s) realloc(p,s)
#define STRDUP(s) strdup(s)		/* comment if strdup is not tracked */
#define FREE(p) free(p)
#define OPEN fopen
#define CLOSE fclose

#define SIZE1 sizeof(int)
#define SIZE2 sizeof(int)*2  /* must be greater than SIZE1 */
#define VERY_LARGE 1000000000  /* size to have alloc fail */

#include <stdio.h>
#include <stdlib.h>

#ifdef RTRT_LONGJMP
#include <setjmp.h>
#endif

#ifdef _WIN32
#include <windows.h>
#endif

#ifdef RTRT_LONGJMP
jmp_buf mark;              /* Address for long jump to jump to */
#endif
char mystring[13],mystring2[15]; /* check non aligned memory */
int *ref_mpk, *miu;

/* this procedure make 1 FFM, 1 FUM, 1 ABWL, 1 FMWL and 1 MAF */
void Error()
{
 int *ffm;
 int *fmwl;
 int p;
 int *fum;
 int *abwl;
 char *maf,*maf1,*maf2,*maf3;

 ffm=ALLOC(SIZE1);
 FREE(ffm);
 /*/ FFM: Freeing Freed Memory */
 FREE(ffm);

 fum=&p;
 /*/ FUM: Freeing Unallocated Memory */
 FREE(fum);

 abwl=CALLOC(2,SIZE1);
 /*/ ABWL: Late Detect Array Bound Write */
 abwl[2]=1;
 FREE(abwl);

 fmwl=ALLOC(SIZE1);
 FREE(fmwl);
 /*/ FMWL: Late Detect Free Memory Write */
 *fmwl=1;
 #ifndef RTRT_NO_MAF
 #pragma attol att_insert _ATT_USER_NOTE("MAF detection may take a while")
 /*/ MAF: Memory Allocation Failure
 / / WARNING, this error might make this appication to crash */
 maf=(char *)ALLOC(VERY_LARGE);
 trace ("very large alloc return %X\n",maf);
 maf1=0;maf2=0;maf3=0;
 if(maf)  maf1=(char *)ALLOC(VERY_LARGE/2);
 if(maf)  trace ("2d very large alloc return %X\n",maf1);
 if(maf1) maf2=(char *)ALLOC(VERY_LARGE/2);
 if(maf1) trace ("3th very large alloc return %X\n",maf2);
 if(maf2) maf3=(char *)ALLOC(VERY_LARGE/2);
 if(maf2) trace ("4th very large alloc return %X\n",maf3);
 FREE(maf);
 if(maf1) FREE(maf1);
 if(maf2) FREE(maf2);
 if(maf3) FREE(maf3);
#endif /* RTRT_NO_MAF */
#ifdef RTRT_LONGJMP
#pragma attol att_insert _ATT_USER_NOTE("longjmp")
 longjmp( mark, -1 );
#endif
}

/* this procedure make 1 MLK, 1 MPK, 1 FIU and 1 MIU */
void Warning()
{
	 int *mlk,**pmlk;
	 int p2;
	 int *mpk;
	#ifdef OPEN
	 FILE *fiu, *nopfile;

	 /*/ FIU: File In Use */
	 fiu=OPEN("foo.txt","w");
	 nopfile=OPEN("foo.log","w");
	 CLOSE(nopfile);
	#endif

	 /*/ MIU: Memory In Use */
	 miu=ALLOC(SIZE1);

	 /*/ MPK: Potential Memory Leak */
	 mpk =ALLOC(SIZE2);
	 ref_mpk=&mpk[1];
	 /*/ ref_mpk is an internal reference for this block (global data, not stack) */
	 mpk=(int *)0;

	 /*/ MLK: Memory Leak */
	 mlk=ALLOC(SIZE1);
	 pmlk=ALLOC(sizeof(void*));
	 *pmlk=mlk;
	 mlk=(int *)0;
	 pmlk=(int **)0;
}

/* this procedure make 4 ABWL and 3 FMWL */
void reallocTest()
{
  int *p;
  int *a,*b,*c,*d;
  int i;
  int num = 5;
  char res=1;

  a = (int*) REALLOC((void*)0,num*SIZE1);
/*/ fulfill data */
  for ( i = 0 ; i < num ; ++i )
    a[i] = i;
/*/ make ABWL */
  a[num] = 10;
  p=a;p--; *p=10;
/*/ re-read data */
  for ( i = 0 ; i < num ; ++i ) {
    trace("a[%d] = %x\n",i,a[i]);
  }
  trace("=== realloc a to b\n");

  b = (int*) REALLOC(a,num*SIZE1*2);
  if (a != b) {
    trace ("realloc to bigger: Pointers are different\n");
    a[2] = 10; /* make FMWL */
  }
/*/fulfill new data */
  for ( i = num ; i < num*2 ; ++i )
    b[i] = i;
/*/ re-read data */
  for ( i = 0 ; i < num*2 ; ++i ) {
    trace("b[%d] = %x\n",i,b[i]);
    if(b[i] != i) res=0;
    }
/*/ make ABWL */
  b[i] = 10;

  c = (int*) REALLOC(b,num*SIZE1);
  if (c != b) {
    trace ("realloc to smaller: Pointers are different\n");
    b[2] = 15; /* make FMWL */
  }
/*/ re-read data */
  for ( i = 0 ; i < num ; ++i ) {
    trace("c[%d] = %x\n",i,c[i]);
    if(c[i] != i) res=0;
  }
  trace("c[size+1] = %x\n",c[i]);
  c[i]=i; /*/ make ABWL */
  trace("the realloc is %s\n",(res ? "OK" : "kO"));
#pragma attol att_insert if (res) _ATT_USER_NOTE("the realloc is OK")
  d = (int*) REALLOC(c,0);
  trace("the realloc (c,0)return %x\n",d);
  c[3] = 10; /*/ make FMWL */
 if(d) d[1]=55; /*/ make ABWL */
}

/* this procedure does not make any error */
void special()
{
	char * m0;
	int *ffm;

    ffm=CALLOC(2,SIZE1);
    FREE(ffm);

    reallocTest();

    m0=ALLOC(0);		/* alloc of 0 sized may return ptr */
	if(m0) FREE(m0);

#ifdef STRDUP
	m0=(char *)STRDUP("12345678");
	if (*m0 != '1') {
#pragma attol att_insert _ATT_USER_NOTE("bad strdup")
     }
     else
     {
#pragma attol att_insert _ATT_USER_NOTE("strdup OK")
     }
    FREE(m0);
#endif
}

int try_it()
{
#ifdef RTRT_LONGJMP
#pragma attol att_insert _ATT_USER_NOTE("setjmp")
 if(setjmp( mark ) ==0)
#endif
	Error();
 Warning();
 return(0);
}

int main()
{
static void *mydeepvar;
size_t s;
static void *mydeepvarbis;

#pragma attol insert RTRT_SIZE_T S; S=0; S--;
s=0;s--;
#pragma attol insert if(sizeof(size_t) == sizeof(RTRT_SIZE_T)) {
#pragma attol insert if(((s < 0) && (S <0)) || (s>0) && (S>0)) {
#pragma attol att_insert _ATT_USER_NOTE("RTRT_SIZE_T ok")
#pragma attol insert } else {
#pragma attol att_insert _ATT_USER_NOTE("RTRT_SIZE_T sign mismatch size_t")
#pragma attol insert }
#pragma attol insert } else {
#pragma attol att_insert _ATT_USER_NOTE("RTRT_SIZE_T size mismatch size_t")
#pragma attol insert }
try_it();
mydeepvar=(void*)ALLOC(10);
mydeepvar=(void*)0xFF;
special();
#pragma attol att_insert _ATT_USER_NOTE("Exit Normally")
exit(0);
return 0;
}
